<?php
class Dreambox
{
	/**
	* @access private
	* @var string
	*/
	private $lIPNumber;
	
	private $lPortNumber;

	/**
	* @access private
	* @var string
	*/
	private $lBoutiques;

	/**
	* @access private
	* @var string
	*/
	private $lType;

	/**
	* @access private
	* @var ProgramGuide
	*/
	private $lProgramGuide;

	/**
	* @access private
	* @var string
	*/
	private $lRecordings;

	/**
	* @access private
	* @var string
	*/
	private $lMovies;

	/**
	* @access private
	* @var array
	*/
	private $lAuthentication;

	private $lOnline;

	/**
	* Constructor. This creates the Dreambox object. If it exists in the session cache, it will be recreated from the cahce instead a new object.
	* @return Dreambox
	*/
	function __construct()
	{
		if (isset($_SESSION["ReStream2.0"]["Dreambox"]) && isset($_SESSION["ReStream2.0"]["Dreambox"]["Obj"])) {
			// Use cached version of the dreambox object
			$lDreamboxObj = unserialize($_SESSION["ReStream2.0"]["Dreambox"]["Obj"]);
			$this->setDreamboxIP($lDreamboxObj->lIPNumber);
			$this->setDreamboxWebPort($lDreamboxObj->lPortNumber);
			$this->setDreamboxType($lDreamboxObj->lType);
			$this->lAuthentication = array();
			$this->lAuthentication = array($lDreamboxObj->lAuthentication[0],$lDreamboxObj->lAuthentication[1]);
			$this->lOnline = $lDreamboxObj->lOnline;
			if (isset($lDreamboxObj->lBoutiques)) {
				$this->lBoutiques = $lDreamboxObj->lBoutiques;
			}
			if (isset($lDreamboxObj->lProgramGuide)) {
				$this->lProgramGuide = $lDreamboxObj->lProgramGuide;
			}
			if (isset($lDreamboxObj->lRecordings)) {
				$this->lRecordings = $lDreamboxObj->lRecordings;
			}
			if (isset($lDreamboxObj->lMovies)) {
				$this->lMovies = $lDreamboxObj->lMovies;
			}
		} else {
			// Initialize new object
			$this->setDreamboxIP(Settings::getDreamboxIP());
			$this->setDreamboxWebPort(Settings::getDreamboxWebPort());
			$this->setDreamboxType(Settings::getEnigmaVersion());
			$this->lAuthentication = array(Settings::getDreamboxUserName(),Settings::getDreamboxPassword());
			$this->lBoutiques = null;
			$this->lRecordings = null;
			$this->lProgramGuide = null;
			$this->lMovies = null;
			$this->lOnline = -1;
			$this->serializeMe();
		}
	}

	/**
	* Save the object in the current session. This is done with serialization
	*/
	private function serializeMe() {
		$_SESSION["ReStream2.0"]["Dreambox"]["cachetime"] = time();
		$_SESSION["ReStream2.0"]["Dreambox"]["Obj"] = serialize($this);
	}

	/**
	* Set the Dreambox IP nummer. If the IP nummer is not valid, it will be ignored.
	* @param string $pIp
	* @return boolean
	*/
	private function setDreamboxIP($pIp)
	{
		$this->lIPNumber = (long2ip(ip2long($pIp)) == $pIp ? $pIp : false);
	}
	
	private function setDreamboxWebPort($pNumber)
	{
		$this->lPortNumber = (is_numeric($pNumber) ? $pNumber : false);
	}

	/**
	* Set the Dreambox type. Valid values are 'enigma1' and 'enigma2'.
	* @param string $pType
	* @return boolean
	*/
	private function setDreamboxType($pType)
	{
		$this->lType = false;
		switch (strtolower($pType)) {
			case "enigma1":
			case "enigma2";
				$this->lType = strtolower($pType);
			break;
		}
	}

	/**
	* Private function to get the authentication part for the Dreambox url
	* @return string
	*/
	private function getAuthentication() {
		$lExtraUrl = "";
		if ($this->lAuthentication[0] != "" || $this->lAuthentication[1] != "") {
			$lExtraUrl = $this->lAuthentication[0] . ":" . $this->lAuthentication[1] . "@";
		}
		return $lExtraUrl;
	}

	/**
	* Check if the Dreambox is online.
	* @return boolean
	*/
	public function isOnline()
	{
		if ((time() - $this->lOnline) > (15 * 60)) { // Cache for 15 minutes
			// recheck
			if (Utils::isWindows()) {
				$lPingData = explode("\n", trim(shell_exec('ping -n 2 ' . $this->lIPNumber)));
				$lArrayPos = sizeof($lPingData)-3;
				$lPingData = explode(",", $lPingData[$lArrayPos]);
				$lSend = substr($lPingData[0],-1);
				$lReceived = substr($lPingData[1],-1);
				//$lLost = substr($lPingData[2],-1);
				$lPingOK = ($lSend == $lReceived);

			} elseif(Utils::isMacOSX()) {
				$lPingData = explode("\n", trim(shell_exec('ping -c 2 -n -q ' . $this->lIPNumber)));
				$lPingLine = explode(",",$lPingData[sizeof($lPingData)-2]);
				$lSendNumber = trim(substr($lPingLine[0],0,2)) * 1;
				$lReceivedNumber = trim(substr($lPingLine[1],0,2)) * 1;

				$lPingOK = ($lSendNumber == $lReceivedNumber);
			} else {
				$lPingData = explode("\n", trim(shell_exec('ping -c 2 -n -q ' . $this->lIPNumber)));
				$lArrayPos = sizeof($lPingData)-2;
				$lPingLine = $lPingData[$lArrayPos];
				$lPingLine = explode(",", $lPingLine);

				$lSentNumber = explode(" ", trim($lPingLine[0]));
				$lSentNumber = $lSentNumber[0];
				$lReturnNumber = explode(" ", trim($lPingLine[1]));
				$lReturnNumber = $lReturnNumber[0];

				$lPingOK = ($lSentNumber == $lReturnNumber);
			}
			if ($lPingOK) $this->lOnline = time();
			else $this->lOnline = -1;
			$this->serializeMe();
			return $lPingOK;
		} else{
			return ($this->lOnline > 0);
		}
	}

	public function canWriteSDPFiles() {
		$lTestFile = getcwd() . "/apps/wowza/content/test.txt";
		if (@touch($lTestFile)) {
			@unlink($lTestFile);
			return true;
		} else {
			return false;
		}
	}

	/**
	* Load the data from the Dreambox. Here all youre boutiques will be collected and saved in the dreambox object.
	*/
	public function loadBoutiques()
	{
		global $gDebugObj;
		if (!$this->isOnline()) return;
		$lChange = false;
		if (!is_array($this->lBoutiques)) {
			$this->lBoutiques = array();
			if ( ($lData = @file_get_contents($this->getBoutiqueUrl())) !== false && ($lData = trim($lData)) != "")
			{
				if (Settings::getDebug()) $gDebugObj->setDebugMessage("Load boutique url: " . $this->getBoutiqueUrl());
				switch ($this->lType) {
					case "enigma1":
						$lStartPos = stripos($lData, "var bouquets = new Array(");
						$lStartPos = stripos($lData, "\n", $lStartPos);
						$lStopPos = stripos($lData, "\n", $lStartPos + 1);
						$lDataNames = explode("\",", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos)));
						$lStartPos = stripos($lData, "var bouquetRefs = new Array(");
						$lStartPos = stripos($lData, "\n", $lStartPos);
						$lStopPos = stripos($lData, "\n", $lStartPos + 1);
						$lDataIDs = explode("\",", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos)));
						for ($i = 0; $i < sizeof($lDataIDs); $i++) {
							$lDataIDs[$i] = trim($lDataIDs[$i]);
							$lDataNames[$i] = trim($lDataNames[$i]);
							if (substr($lDataIDs[$i],0,1)   == "\"") $lDataIDs[$i]   = substr($lDataIDs[$i],1);
							if (substr($lDataIDs[$i],-1)    == "\"") $lDataIDs[$i]   = substr($lDataIDs[$i],0,-1);
							if (substr($lDataNames[$i],0,1) == "\"") $lDataNames[$i] = substr($lDataNames[$i],1);
							if (substr($lDataNames[$i],-1)  == "\"") $lDataNames[$i] = substr($lDataNames[$i],0,-1);
							$this->lBoutiques[] = new Boutique($lDataIDs[$i], $lDataNames[$i]);
							$lChange = true;
						}
					break;
					case "enigma2";
						$lXMLBoutique = new DOMDocument();
						$lXMLBoutique->loadXML($lData);
						$lServices = $lXMLBoutique->getElementsByTagName("e2service");
						foreach ($lServices as $lService) {
							$lBoutique["e2servicereference"] = str_replace("\\", "", $lService->getElementsByTagName("e2servicereference")->item(0)->nodeValue);
							// Suhosin Patch wierdness....?
							if (strpos("&quot;", $lBoutique["e2servicereference"]) !== false) {
								$lBoutiqueId = explode("&quot;", $lBoutique["e2servicereference"]);
							} else {
								$lBoutiqueId = explode("\"", $lBoutique["e2servicereference"]);
							}
							$lBoutiqueId = $lBoutiqueId[1];
							$this->lBoutiques[] = new Boutique(
								$lBoutiqueId,
								$lService->getElementsByTagName("e2servicename")->item(0)->nodeValue
							);
						}
						$lChange = true;
					break;
				}
			}
		}
		if ($lChange) $this->serializeMe();
	}

	/**
	* Load the channel data. This is done after the boutiques are loaded. The actual loading of the data happens in the Boutique class.
	*/
	public function loadChannels()
	{
		global $gDebugObj;
		if (!$this->isOnline()) return;
		if (!isset($this->lBoutiques) || !is_array($this->lBoutiques)) return;
		$lChange = false;
		foreach ($this->lBoutiques as $lBoutique) {
			if ($lBoutique->getChannelCount() == 0) {
				if ( ($lData = @file_get_contents($this->getChannelUrl($lBoutique->getID()))) !== false && ($lData = trim($lData)) != "")
				{
					if (Settings::getDebug()) $gDebugObj->setDebugMessage("Load channel url: " . $this->getChannelUrl($lBoutique->getID()));
					switch ($this->lType) {
						case "enigma1":
							$lStartPos = stripos($lData,"var bouquetRefs = new Array(");
							$lStartPos = stripos($lData,"\n",$lStartPos);
							$lStopPos = stripos($lData,"\n",$lStartPos+1);
							$lDataIDs = explode("\",", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos)));
							for ($i = 0; $i < sizeof($lDataIDs); $i++) {
								$lDataIDs[$i] = trim($lDataIDs[$i]);
								if (substr($lDataIDs[$i],0,1)  == "\"") $lDataIDs[$i] = substr($lDataIDs[$i],1);
								if (substr($lDataIDs[$i],-1) == "\"") $lDataIDs[$i] = substr($lDataIDs[$i],0,-1);
								if (trim($lBoutique->getID()) == trim($lDataIDs[$i])) { // This is me....
									$lStartPos = stripos($lData,"channels[$i] = new Array(");
									if ($lStartPos !== false) {
										$lStartPos += strlen("channels[$i] = new Array(");
										$lStopPos = stripos($lData,");",$lStartPos);
										$lDataNames = explode("\",", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos)));
										$lStartPos = stripos($lData,"channelRefs[$i] = new Array(");
										if ($lStartPos !== false) {
											$lStartPos += strlen("channelRefs[$i] = new Array(");
											$lStopPos = stripos($lData,");",$lStartPos);
											$lDataIDs2 = explode("\",", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos)));
											for ($j = 0; $j < sizeof($lDataIDs2); $j++) {
												$lDataIDs2[$j] = trim($lDataIDs2[$j]);
												$lDataNames[$j] = trim($lDataNames[$j]);
												if (substr($lDataIDs2[$j],0,1)  == "\"") $lDataIDs2[$j]  = substr($lDataIDs2[$j],1);
												if (substr($lDataIDs2[$j],-1)   == "\"") $lDataIDs2[$j]  = substr($lDataIDs2[$j],0,-1);
												if (substr($lDataNames[$j],0,1) == "\"") $lDataNames[$j] = substr($lDataNames[$j],1);
												if (substr($lDataNames[$j],-1)  == "\"") $lDataNames[$j] = substr($lDataNames[$j],0,-1);
												$lBoutique->addChannel(new Channel($lDataIDs2[$j],$lDataNames[$j]));
												$lChange = true;
											}
										}
									}
									break; // Stop looping, because current Boutique is found and the channels are loaded
								}
							}
						break;

						case "enigma2";
							$lXMLChannel = new DOMDocument();
							$lXMLChannel->loadXML($lData);
							$lChannels = $lXMLChannel->getElementsByTagName("e2service");
							foreach ($lChannels as $lChannel) {
								$lBoutique->addChannel(
									new Channel(
										$lChannel->getElementsByTagName("e2servicereference")->item(0)->nodeValue,
										$lChannel->getElementsByTagName("e2servicename")->item(0)->nodeValue
									)
								);
								$lChange = true;
							}
						break;
					}
				}
			}
		}
		if ($lChange) $this->serializeMe();
	}

	public function loadProgramGuide($pChannelID = null) {
		//if (!$this->isOnline()) return;
		$lChange = false;
		foreach ($this->lBoutiques as $lBoutique) {
			if (!isset($this->lProgramGuide[$lBoutique->getID()])) $this->lProgramGuide[$lBoutique->getID()] = array();
			foreach ($lBoutique->getChannels() as $lChannel) {
				if (($pChannelID == null || $pChannelID == $lChannel->getID()) && !isset($this->lProgramGuide[$lBoutique->getID()][$lChannel->getID()]) ) {
					if ($this->isOnline()) {
						$this->lProgramGuide[$lBoutique->getID()][$lChannel->getID()] = $this->loadProgramGuideChannel($lChannel);
						$lChange = true;
					}
				}
			}
		}
		if ($lChange) $this->serializeMe();
	}

	public function getProgramGuide($pChannelID) {
		foreach ($this->lBoutiques as $lBoutique) {
			foreach ($lBoutique->getChannels() as $lChannel) {
				if ($pChannelID == $lChannel->getID() && isset($this->lProgramGuide[$lBoutique->getID()][$lChannel->getID()]))return $this->lProgramGuide[$lBoutique->getID()][$lChannel->getID()];
			}
		}
		return false;
	}

	public function loadProgramGuideChannel($pChannel) {
		global $gDebugObj;
		if (!$this->isOnline()) return;
		$lGuide = new ProgramGuide($pChannel);
		if (!$pChannel->isMarker() && ($lData = @file_get_contents($this->getProgramGuideUrl($pChannel->getID()))) !== false && ($lData = trim($lData)) != "")
		{
			switch ($this->lType) {
				case "enigma1":
					$lData = explode("<tr",$lData);
					$this->lProgramGuide[$this->lChannelID] = array();
					for ($i = 2; $i < sizeof($lData); $i++) {
						$lStartPos = stripos($lData[$i],"javascript:record(") + strlen("javascript:record(");
						$lStopPos = stripos($lData[$i],")",$lStartPos+1);
						$lDataParts = explode(",",str_ireplace("'","",trim(substr($lData[$i],$lStartPos,$lStopPos-$lStartPos))));
						$lStartPos = stripos($lData[$i],"class=\"description\"");

						if ($lStartPos !== false) {
							$lStartPos += strlen("class=\"description\"")+1;
							$lStopPos = stripos($lData[$i],"</",$lStartPos+1);
							$lExtendedInfo = trim(substr($lData[$i],$lStartPos,$lStopPos-$lStartPos));
						}
						$lGuide->addProgram(new Program(trim($lDataParts[1]),trim($lDataParts[1]),trim($lDataParts[2]),trim($lDataParts[3]),$lExtendedInfo,""));
					}
				break;
				case "enigma2";
					$lXMLEPG = new DOMDocument();
					$lXMLEPG->loadXML($lData);
					$lPrograms = $lXMLEPG->getElementsByTagName("e2event");
					foreach ($lPrograms as $lProgram) {
						$lGuide->addProgram(
							new Program(
								$lProgram->getElementsByTagName("e2eventid")->item(0)->nodeValue,
								$lProgram->getElementsByTagName("e2eventstart")->item(0)->nodeValue,
								$lProgram->getElementsByTagName("e2eventduration")->item(0)->nodeValue,
								$lProgram->getElementsByTagName("e2eventtitle")->item(0)->nodeValue,
								$lProgram->getElementsByTagName("e2eventdescription")->item(0)->nodeValue,
								$lProgram->getElementsByTagName("e2eventdescriptionextended")->item(0)->nodeValue
							)
						);
					}
				break;
			}
			$lGuide->sortOnStartTime();
		} else {
			//echo "Error getting EPG: " . $this->getProgramGuideUrl($pChannel->getID()) . "\n";
		}
		return $lGuide;
	}

	/**
	* Get the Boutique url. This is used for getting boutique information from your dreambox.
	* @return string
	*/
	private function getBoutiqueUrl()
	{
		switch ($this->lType) {
			case "enigma1":
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/body?mode=zap&zapmode=0&zapsubmode=4";
			break;
			case "enigma2";
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/web/getservices?sRef=1:7:1:0:0:0:0:0:0:0:FROM%20BOUQUET%20%22bouquets.tv%22%20ORDER%20BY%20bouquet";
			break;
			default:
				return false;
			break;
		}
	}

	/**
	* Get the Boutique url. This is used for getting boutique information from your dreambox.
	* @return string
	*/
	private function getChannelUrl($pBoutiqueID)
	{
		switch ($this->lType) {
			case "enigma1":
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/body?mode=zap&zapmode=0&zapsubmode=4";
			break;
			case "enigma2";
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/web/getservices?sRef=1:7:1:0:0:0:0:0:0:0:FROM%20BOUQUET%20%22" . $pBoutiqueID . "%22%20ORDER%20BY%20bouquet";
			break;
		}
	}

	private function getProgramGuideUrl($pChannelID)
	{
		switch ($this->lType) {
			case "enigma1":
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/getcurrentepg?type=extended&ref="  . $pChannelID;
			break;
			case "enigma2";
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/web/epgservice?sRef="  . $pChannelID;
			break;
		}
	}

	public function getBoutiques()
	{
		return $this->lBoutiques;
	}

	public function findChannel($pChannelID) {
		foreach ($this->lBoutiques as $lBoutique) {
			if (($lChannelObj = $lBoutique->getChannel($pChannelID)) !== false) {
				return $lChannelObj;
			}
		}
		return false;
	}
	/**
	* Get the Recording url. This is used for getting recoring information from your dreambox.
	* @return string
	*/

	private function getRecordingUrl()
	{
		switch ($this->lType) {
			case "enigma1":
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/body?mode=zap&zapmode=3&zapsubmode=1";
			break;
			case "enigma2";
				return "http://" . $this->getAuthentication() . $this->lIPNumber . ":" . $this->lPortNumber . "/web/movielist?tag=";
			break;
		}
	}

	/**
	* Get all recordings from the dreambox. The recordings will be save in a recording object array.
	* It can self see with enigma version is used, and what the code will be that will be returned from the dreambox
	*/
	public function loadRecordings()
	{
		if (!$this->isOnline()) return;
		$lChange = false;
		if (!is_array($this->lRecordings) || count($this->lRecordings) == 0) {
			$this->lRecordings = array();
			if ( ($lData = @file_get_contents($this->getRecordingUrl())) !== false && ($lData = trim($lData)) != "")
			{
				switch ($this->lType) {
					case "enigma1":
						$lStartPos = stripos($lData, "channels[0] = new Array(\"") + strlen("channels[0] = new Array(\"");
						$lStopPos = stripos($lData, "\");", $lStartPos + 1);
						$lDataNames = explode(",", str_ireplace("\"", "", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos))));

						$lStartPos = stripos($lData, "channelRefs[0] = new Array(\"") + strlen("channelRefs[0] = new Array(\"");
						$lStopPos = stripos($lData, "\");", $lStartPos + 1);
						$lDataIDs = explode(",", str_ireplace("\"", "", trim(substr($lData, $lStartPos, $lStopPos - $lStartPos))));

						for ($i = 0; $i < sizeof($lDataIDs); $i++) {
							$lDataLine = trim($lDataNames[$i]);
							$lTitle = trim(substr($lDataLine, stripos($lDataLine, "]") + 1));
							$lFileSize = substr($lDataLine, 1, stripos($lDataLine, "]") - 4) * 1024;
							$lFileDate = explode("-", trim(substr($lDataLine, stripos($lDataLine, "]") + 1, 9)));
							if (sizeof($lFileDate) == 3) {
								$lFileDate = mktime(0, 0, 0, $lFileDate[1], $lFileDate[2], $lFileDate[0]);
								$lTitle = trim(substr($lTitle, 11));
							} else {
								$lFileDate = time();
							}
							$this->lRecordings[] = new Recording($lDataIDs[$i], $lTitle, "", "", $lFileDate, "", $lDataIDs[$i], $lFileSize);
							$lChange = true;
						}
					break;

					case "enigma2";
						$lXMLRecoring = new DOMDocument();
						$lXMLRecoring->loadXML($lData);
						$lRecordinds = $lXMLRecoring->getElementsByTagName("e2movie");
						foreach ($lRecordinds as $lRecording) {
							$this->lRecordings[] = new Recording(
								$lRecording->getElementsByTagName("e2servicereference")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2title")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2description")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2servicename")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2time")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2length")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2filename")->item(0)->nodeValue,
								$lRecording->getElementsByTagName("e2filesize")->item(0)->nodeValue
							);
							$lChange = true;
						}
					break;
				}
				$this->sortOnStartTime();
			}
		}
		if ($lChange) $this->serializeMe();
	}

	public function loadMovies()
	{
		$lChange = false;
		if (!is_array($this->lMovies) || count($this->lMovies) == 0) {
			$this->lMovies = array();
			if (Settings::getMoviesPath() != "") {
				$lMovieStartLocation = new RecursiveDirectoryIterator(Settings::getMoviesPath());
				foreach (new RecursiveIteratorIterator($lMovieStartLocation) as $lMovieFile) {
					$lMovieFile = pathinfo($lMovieFile);
					if (in_array(strtolower($lMovieFile["extension"]),Settings::getValidMovies())) {
						$lMoviePath = trim($lMovieFile["dirname"] . "/" . $lMovieFile["basename"]);
						if ($lMoviePath != "") {
							$this->lMovies[] = new Movie($lMoviePath);
							$lChange = true;
						}
					}
				}
				$this->sortOnName();
			} else {
				$lChange = true;
			}
		}
		if ($lChange) $this->serializeMe();
	}

	public function getMovie($pMovieID) {
		foreach ($this->lMovies as $lMovie) {
			if ($pMovieID == $lMovie->getID()) return $lMovie;
		}
		return false;
	}

	public function loadMovie($pMovieID) {
		if (($lMovieObj = $this->getMovie($pMovieID)) !== false) {
			if ($lMovieObj->getDuration() == -1) { // Only load movie data when not loaded before...
				$lMovieObj->loadInfo();
				$this->serializeMe();
			}
		}
	}

	/**
	* Sort the programs based in start time
	*/
	private function sortOnStartTime() {
		$lStartTimes = array();
		foreach ($this->lRecordings as $lRecording) {
			$lStartTimes[] = $lRecording->getStartTime();
		}
		array_multisort($lStartTimes, SORT_ASC, $this->lRecordings);
	}


	public function getRecordings()
	{
		return $this->lRecordings;
	}

	public function findRecording($pRecordingID) {
		foreach ($this->lRecordings as $lRecording) {
			if ($lRecording->getID() == $pRecordingID) return $lRecording;
		}
		return false;
	}

	private function sortOnName() {
		$lNames = array();
		foreach ($this->lMovies as $lMovie) {
			$lNames[] = $lMovie->getName();
		}
		array_multisort($lNames, SORT_ASC, $this->lMovies);
	}

	public function getMovies()
	{
		return $this->lMovies;
	}

	public function getStreamURL($pChannelID)
	{
		global $gDebugObj;
		if (Settings::getMoviesPath() != "" && substr($pChannelID,0,strlen(Settings::getMoviesPath())) == Settings::getMoviesPath()) {
			$lStreamUrl = $pChannelID;
		} else {
			switch ($this->lType) {
				case "enigma1":
					if (substr($pChannelID,-2) == "ts") { # Recording
						$lRecording = explode(":",$pChannelID);
						$lStreamUrl = "http://" . $this->getAuthentication() . Settings::getDreamboxIP() .  ":31342". $this->dreamboxStreamUrlFormat($lRecording[10]);
					} else{
						//Zap first
						$lZapUrl = "http://" . $this->getAuthentication() . Settings::getDreamboxIP() . ":" . $this->lPortNumber . "/cgi-bin/zapTo?path=". $pChannelID ."&curBouquet=0&curChannel=0";
						if (Settings::getDebug()) $gDebugObj->setDebugMessage("VLC ZAP Enigma1 Url: '".$lZapUrl . "'");
						$Dummy = @file_get_contents($lZapUrl);
						sleep(1); // Wait for the dreambox to change channel
						$lStreamUrl = "http://" . $this->getAuthentication() . Settings::getDreamboxIP() . ":" . $this->lPortNumber . "/video.m3u";
//						$lStreamUrl = trim(file_get_contents("http://" . $this->getAuthentication() . Settings::getDreamboxIP() . "/video.m3u"));
						if (Settings::getDebug()) $gDebugObj->setDebugMessage("VLC Final Enigma1 Url: '".$lStreamUrl . "'");
					}
				break;

				case "enigma2":
					if (substr($pChannelID,-3) == ".ts") {
						$pChannelID = substr($pChannelID,stripos($pChannelID,"/"));
						$lStreamUrl = "http://" . $this->getAuthentication() . Settings::getDreamboxIP() .  ":" . $this->lPortNumber . "/file?file=" . $this->dreamboxStreamUrlFormat($pChannelID);
					} else {
						$lStreamUrl = "http://" . $this->getAuthentication() . Settings::getDreamboxIP() .  ":8001/" . $pChannelID;
					}
					if (Settings::getDebug()) $gDebugObj->setDebugMessage("VLC Final Enigma2 Url: '".$lStreamUrl . "'");
				break;
			}
		}
		return $lStreamUrl;
	}
	
	public function getCurrentProgram () {
		// Enigma2 url: http://192.168.5.119/web/getcurrent
	}
	
	private function dreamboxStreamUrlFormat($pUrl) {
		$pUrl = trim($pUrl);
		switch ($this->lType) {
			case "enigma1":
			//break;
			case "enigma2":
				return str_replace(array(" "),array("%20"),$pUrl);
			break;
		}
	}

	/**
	* Dump the complete object to screen.
	*/
	public function dump()
	{
		print_r($this);
	}
}
?>